"use strict";

var _nanomatch = _interopRequireDefault(require("nanomatch"));

var _pundleApi = require("pundle-api");

var _package = _interopRequireDefault(require("../package.json"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } } function _next(value) { step("next", value); } function _throw(err) { step("throw", err); } _next(); }); }; }

const DEFAULT_EXCLUDE = ['node_modules/**'];
const ALLOWED_VERSIONS = [6, 7];

function createComponent({
  exclude = DEFAULT_EXCLUDE,
  processOutsideProjectRoot,
  options = {},
  version
} = {}) {
  if (!ALLOWED_VERSIONS.includes(version)) {
    throw new Error(`options.version is required to be one of: ${ALLOWED_VERSIONS.join(', ')} got: ${version}`);
  }

  return (0, _pundleApi.createFileTransformer)({
    name: 'pundle-transformer-babel',
    version: _package.default.version,
    priority: 1250,

    callback({
      file,
      context
    }) {
      return _asyncToGenerator(function* () {
        if (file.format !== 'js') return null;

        if (exclude.some(item => _nanomatch.default.isMatch(file.filePath, item))) {
          // Excluded
          return null;
        }

        if (!processOutsideProjectRoot && !file.filePath.startsWith(context.config.rootDirectory)) {
          // Outside project root
          return null;
        }

        let babelToLookFor = null;

        if (version === 6) {
          babelToLookFor = 'babel-core';
        } else if (version === 7) {
          babelToLookFor = '@babel/core';
        }

        const _loadLocalFromContext = (0, _pundleApi.loadLocalFromContext)(context, babelToLookFor ? [babelToLookFor] : []),
              name = _loadLocalFromContext.name,
              exported = _loadLocalFromContext.exported;

        if (!name) {
          throw new Error(`Babel not found in '${context.config.rootDirectory}' (tried ${babelToLookFor || ''})`);
        }

        const transformed = yield new Promise((resolve, reject) => {
          const response = exported.transform(file.contents, _objectSpread({
            babelrc: true,
            filename: file.filePath,
            sourceMaps: true,
            sourceType: 'module',
            highlightCode: false,
            sourceFileName: file.filePath
          }, version === 7 ? {
            root: context.config.rootDirectory
          } : {}, options), function (err, result) {
            if (err) {
              reject(err);
            } else resolve(result);
          });

          if (version === 6) {
            resolve(response);
          }
        });
        return {
          contents: transformed.code,
          sourceMap: JSON.stringify(transformed.map)
        };
      })();
    }

  });
}

module.exports = createComponent;